#! /usr/bin/env node
'use strict';

var fs = require('fs');

var _require = require('path'),
    resolve = _require.resolve,
    dirname = _require.dirname;

var _ = require('lodash');
var commander = require('commander');

var _require2 = require('chalk'),
    magenta = _require2.magenta;

var getChartData = require('../chartData');
var Logger = require('../Logger');

var program = commander.version(require('../../package.json').version).usage('<bundleStatsFile> [bundleDir] [options]\n\n  Arguments:\n\n    bundleStatsFile  Path to Webpack Stats JSON file.\n    bundleDir        Directory containing all generated bundles.\n                     You should provided it if you want analyzer to show you the real parsed module sizes.\n                     By default a directory of stats file is used.').option('-m, --mode <mode>', 'Analyzer mode. Should be `server` or `static`.' + br('In `server` mode analyzer will start HTTP server to show bundle report.') + br('In `static` mode single HTML file with bundle report will be generated.') + br('Default is `server`.'), 'server').option('-p, --port <n>', 'Port that will be used in `server` mode to start HTTP server.' + br('Default is 8888.'), Number, 8888).option('-r, --report <file>', 'Path to bundle report file that will be generated in `static` mode.' + br('Default is `report.html`.'), 'report.html').option('-O, --no-open', "Don't open report in default browser automatically.").option('-R, --reporter <package name>', 'The package to use for reporting').parse(process.argv);

var mode = program.mode,
    port = program.port,
    reportFilename = program.report,
    reporter = program.reporter,
    openBrowser = program.open,
    _program$args = program.args,
    bundleStatsFile = _program$args[0],
    bundleDir = _program$args[1];


if (!bundleStatsFile) showHelp('Provide path to Webpack Stats file as first argument');
if (mode !== 'server' && mode !== 'static') showHelp('Invalid mode. Should be either `server` or `static`.');
if (mode === 'server' && isNaN(port)) showHelp('Invalid port number');
if (reporter) {
  reporter = require(reporter);
} else {
  reporter = require('webpack-bundle-analyzer-reporter-treemap');
}

bundleStatsFile = resolve(bundleStatsFile);

if (!bundleDir) bundleDir = dirname(bundleStatsFile);

var bundleStats = void 0;
try {
  bundleStats = readStatsFromFile(bundleStatsFile);
} catch (err) {
  console.error('Could\'t read webpack bundle stats from "' + bundleStatsFile + '":\n' + err);
  process.exit(1);
}

var logger = new Logger();
var chartData = getChartData({ logger: logger, bundleStats: bundleStats, bundleDir: bundleDir });

if (!chartData) {
  // TODO: Exit with an error code as the only reason `chartData` is falsy
  // at this point is if `getChartData` had an error
  process.exit(0);
}

if (mode === 'server') {
  reporter.startServer(chartData, {
    openBrowser: openBrowser,
    port: port,
    bundleDir: bundleDir,
    logger: logger
  });
} else {
  reporter.generateReport(chartData, {
    openBrowser: openBrowser,
    reportFilename: resolve(reportFilename),
    bundleDir: bundleDir,
    logger: logger
  });
}

function showHelp(error) {
  if (error) console.log('\n  ' + magenta(error));
  program.outputHelp();
  process.exit(1);
}

function br(str) {
  return '\n' + _.repeat(' ', 21) + str;
}

function readStatsFromFile(filename) {
  return JSON.parse(fs.readFileSync(filename, 'utf8'));
}